//HavokLevel.cpp
#include <Demos/demos.h>
#include "HavokLevel.h"

#include <stdexcept>
#include <Physics/Dynamics/World/hkpWorld.h>
#include <Common/Serialize/Util/hkRootLevelContainer.h>
#include <Common/Serialize/Util/hkLoader.h>
#include <Physics/Utilities/Serialize/hkpPhysicsData.h>
#include <Physics/Dynamics/Entity/hkpRigidBody.h>
#include <Physics/Utilities/Collide/ShapeUtils/ShapeScaling/hkpShapeScalingUtility.h>
#include <Physics/Utilities/Dynamics/ScaleSystem/hkpSystemScalingUtility.h>

#include "HavokManager.h"


HavokLevel* HavokLevel::m_pInstance = 0L;


bool HavokLevel::CreateInstance()
{
	if( m_pInstance )
		return false;

	m_pInstance = new HavokLevel();
	bool bRes = m_pInstance->Init();
	return bRes;
}
//--------------------------------------------------------------------------------
void HavokLevel::ReleaseInstance()
{
	if( m_pInstance == 0L )
		return;
	
	m_pInstance->UnInit();
	delete m_pInstance;
	m_pInstance = 0L;
}
HavokLevel::HavokLevel()
{

}
//------------------------------------------------------------------------------------------------
HavokLevel::~HavokLevel()
{

}
//------------------------------------------------------------------------------------------------
void HavokLevel::UnInit()
{
	//⼭ m_rigidVec ص  
	hkpWorld* pWorld = HavokManager::Instance()->GetWorld();
	pWorld->lock();
	int nCount = (int)m_rigidVec.size();

	for( int n = 0; n < nCount; ++n )
	{
		pWorld->removeEntity( m_rigidVec[n] );
	}
	pWorld->unlock();

	m_rigidVec.clear();
}
//------------------------------------------------------------------------------------------------
hkpRigidBody* HavokLevel::AddWorld( const char* fileName )
{
	hkLoader* loader = HavokManager::Instance()->GetLoader();
	hkRootLevelContainer* container = loader->load( fileName );

	if(!container)
	{
		throw std::runtime_error("Can't load collision mesh ");
	}

	HK_ASSERT2(0xaefe9356, container != HK_NULL , "Could not load asset");
	hkpPhysicsData* physics = static_cast<hkpPhysicsData*>( container->findObjectByType( hkpPhysicsDataClass.getName() ) );
	HK_ASSERT2(0x245982ae, physics != HK_NULL, "Could not find physics data in root level object" );

	hkpWorld* pWorld = HavokManager::Instance()->GetWorld();
	pWorld->markForWrite();

	for ( int i = 0; i < physics->getPhysicsSystems().getSize(); ++i )
	{
		hkpPhysicsSystem* system = physics->getPhysicsSystems()[i];

#if  0
		hkpSystemScalingUtility::scaleSystem( system, 0.5f );
#endif

		for( int k = 0; k < system->getRigidBodies().getSize(); ++k )
		{
			//浹 ޽ 
			hkpRigidBody* rigidBody = system->getRigidBodies()[k];
			hkpRigidBodyCinfo cInfo;
			rigidBody->getCinfo( cInfo );
			hkpShape* shape = (hkpShape*)cInfo.m_shape;
			hkpShapeScalingUtility::scaleShape( shape, 0.5f );

#if   0
			hkTransform trans = rigidBody->getTransform();
			trans( 0, 0 ) = 0.5f;
			trans( 1, 1 ) = 0.5f;
			trans( 2, 2 ) = 0.5f;
			rigidBody->setTransform( trans );
#endif
			pWorld->addEntity( rigidBody );
			rigidBody->removeReference();
		}
	}


	//hkpRigidBody* pRigidBody = physics->getPhysicsSystems()[0]->getRigidBodies()[0];
	//pRigidBody->setPosition(hkVector4(0,0,0));

	pWorld->unmarkForWrite();
	return NULL;
}
//------------------------------------------------------------------------------------------------
hkpPhysicsData* HavokLevel::AddPhysicsSystem( const char* fileName )
{
	hkLoader* loader = HavokManager::Instance()->GetLoader();
	hkRootLevelContainer* container = loader->load( fileName );
	if(!container)
	{
		throw std::runtime_error("Can't load collision mesh ");
	}

	HK_ASSERT2(0xaefe9356, container != HK_NULL , "Could not load asset");
	hkpPhysicsData* physics = static_cast<hkpPhysicsData*>( container->findObjectByType( hkpPhysicsDataClass.getName() ) );
	HK_ASSERT2(0x245982ae, physics != HK_NULL, "Could not find physics data in root level object" );

	hkpWorld* pWorld = HavokManager::Instance()->GetWorld();
	pWorld->lock();

	//ġ ̵
	for ( int i = 0; i < physics->getPhysicsSystems().getSize(); ++i )
	{
		hkpPhysicsSystem* system = physics->getPhysicsSystems()[i];
		for( int k = 0; k < system->getRigidBodies().getSize(); ++k )
		{
			//浹 ޽ 
			hkpRigidBody* rigidBody = system->getRigidBodies()[k];
			hkpRigidBodyCinfo cInfo;
			rigidBody->getCinfo( cInfo );
			hkpShape* shape = (hkpShape*)cInfo.m_shape;
			hkpShapeScalingUtility::scaleShape( shape, 0.5f );
		}
	}

	// Add all the physics systems to the world
	for ( int i = 0; i < physics->getPhysicsSystems().getSize(); ++i )
	{
		pWorld->addPhysicsSystem( physics->getPhysicsSystems()[i] );
	}

	pWorld->unlock();
	return physics;
}
//------------------------------------------------------------------------------------------------


//------------------------------------------------------------------------------------------------